/**************************************************************

   Program Title:  SmartLCD Controller
   
   Program Name:  SmartLCD_Controller.c	  Version: 0.89
   
   Author:  SS	   					  Date:  November 9, 2010
   
  This program monitors data on SCL/SDA or UART and writes it to the ECMA1007
  LCD panel. A simple command structure is used to write text to the 
  screen.  The LCD panel is addressed by comparing addresses stored 
  in EEPROM with commands sent on the 2-Wire line.
  
  The SmartLCD PCB has pads for a xtal or clock osc for real time
  clock or precision operation.
  
  The SmartLCD PCB also has a pad for the ST Electronics three axis 
  programmable accelerometer.
  
  I2C default address is:  0x25
  
  Pin assignments:
    PC4  Pin 1 - SDA
    PC5  Pin 3 - SCL
    PC3  Pin 20 - SCL ECMA1007 
    PC2  Pin 11 - SI ECMA1007
    PB1  Pin 4 - CS ECMA1007
    PB2  Pin 8 - LA0 ECMA1007
    PB0  Pin 7 - RESET ECMA1007

  Settings can be stored in EEPROM for later recall at power up.

  EEPROM Settings:
     Address    Default    Function
       0x00       0x01      Address of Controller - 1-256, 0 broadcast
       0x05       0x00      Oscillator calibrated - 0 - No
       0x06       0x00      Oscillator calibration value
       0x07       0x00      

  2-Wire Commands:  Commands are sent to the display controller using
  a TWI protocol.  Wakeup address and selection addresses are stored
  in EEPROM and can be changed.

     Command      Function
       DAC        display character at address A
       DC         Display character at current cursor location
       C          Clear screen
       S          Scroll up one line
       NA         New selection address
       WA         Wakeup address
       H          Turn IR High power on

   Fuse settings should be:  F9 DF E2
   
   Copyright 2004-2010, SOC Robotics, Inc.  All rights reserved.
   
****************************************************************/
/*
 * Copyright 2004-2010, SOC Robotics, Inc.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the copyright holders nor the names of
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 * 4. Redistribution is only allowed in combination with SOC Machines, 
 *    Inc. hardware products.
 *
 * THIS SOFTWARE IS PROVIDED BY SOC ROBOTICS, INC. AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL OC MACHINES, 
 * INC. OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * For additional information see http://www.soc-robotics.com
 */

/*
 * $Log: SmartLCD_Controller.c,v $
 * Revision 0.89  2006/02/07 16:40:20  SS
 *    Arduino integration, 19200baud, USART active
 *    real time clock 
 * Revision 0.86  2006/02/07 16:40:20  SS
 *    2-wire interface, 8MHz clock, Serial Flash writer, XON/XOFF, 9,600baud
 *    real time clock and auto-internal osc calibration.
 * Revision 0.52  2005/06/04 18:36:42  SS
 * Preview release
 * 
 */ 
#include <iom168v.h>
#include <macros.h>
#include <string.h>
#include <stdlib.h>
#include <math.h>
//#include <stdio.h>
#include "SmartLCD_Uart.h"
#include "SmartLCD_Defines.h"
#include "SmartLCD_Extern.h"
//#include "USI_TWI_Slave.h"

//  Variable declarations
unsigned int intelsuperaddress,oldpage,oldbyteoffset,totalrxcnt;
unsigned int rtccountdown1,rtccountdown2,rtccountdown3,rtccountdown4;
unsigned int mseccountdown1,mseccountdown2,mseccountdown3,mseccountdown4;
unsigned int seconds,minutes,hours,days,count;

unsigned int totalbytecnt,xoncnt,xoffcnt;
unsigned char frameerror,dataoverrun,numstrg[9],lcdchiptype;
unsigned char lcdstate,rtcclockcnt,textdisplaymode;

extern int _textmode;

/* Static Variables */
unsigned char UART_RxBuf[UART_RX_BUFFER_SIZE];
unsigned char UART_RxHead;
unsigned char UART_RxTail;
unsigned char UART_TxBuf[UART_TX_BUFFER_SIZE];
unsigned char UART_TxHead;
unsigned char UART_TxTail;
char  xoffsent, xonsent, enablexonoff, echoinchars,sendxoff;
char  rxcnt,checkflag,rxcntmax,flip2;
unsigned char  instrg[8],vlow,vhigh,hexbyte[3],lcdledpower;

unsigned char FlashAddress[3];
const char signon1[] = "\n\r\n\rSmartLCD Controller V0.89\n\r";
const char signon11[] = "(c) Copyright SOC Robotics, Inc.\n\r";
const char signon12[] = "Sending ESC c turns character display on\n\r\n\r";

// Function declarations
unsigned char EEPROMwrite(int,unsigned char);
unsigned char EEPROMread(int);

// Sample TWI transmission commands
#define TWI_CMD_MASTER_WRITE 0x00
#define TWI_CMD_MASTER_READ  0x01

// TBD - how many bytes can be transmitted per 2-wire operation?
#define MESSAGEBUF_SIZE       4

//  Set default slave address
#define    TWI_SLAVEADDRESS		 0x26

// Clear display - move cursor to home
#define    LCDCLR	0x01

// E1010 and E1007 Commands
#define	   LCDCHIPE1010 0x10
#define    LCDCHIPE1007 0x07   

// move cursor to home position command
#define    LCDHOME   	0x10  

// Static Display Command
//   D1/0 - 0,0  Display OFF
//   D1/0 - 0,1  1-2Hz blink
//   D1/0 - 1,0  3-4Hz blink
//   D1/0 - 1,1  Display ON
#define    LCDSTATICDISPLAY	0x20 
#define    LCDDISPLAYON  	0x03
#define    LCDDISPLAYOFF	0x00 

// Display control command
//   D3-C, D2-B, D1-DC, D0-D
//   C  -> 1 cursor on, 0 off
//   DC -> 1 double cursor on, 0 off
//   B  -> 1 blink on, 0 off
//   D  -> 1 display on, 0 off	 
#define    LCDDISPLAY          0x30 
#define    LCDCURSORON	       0x08
#define    LCDCURSOROFF        0x00
#define    LCDCURSORBLINK      0x04 
#define    LCDCURSONNOBLINK    0x00
#define    LCDCURSORDISPLAYON  0x01
#define    LCDCURSORDISPLAYOFF 0x00

// Power Save Command
//   D1-O, D0-PS
//   PS -> 1 power save on, 0 off
//   O  -> 1 oscillator circuit on, 0 off
#define	   LCDPOWERSAVE    0x40
#define    LCDPOWERSAVEON  0x01
#define    LCDPOWERSAVEOFF 0x00
#define    LCDOSCON		   0x02
#define    LCDOSCOFF	   0x00

// Power supply control
//   D2-VC, D1-VF, D0-P
//   VC -> 1 voltage regulator on, 0 off
//   VF -> 1 voltage follower on, 0 off
//   P  -> 1 voltage booster on, 0 off
#define    LCDPOWERSUPPLY   0x50
#define	   LCDVOLTREGON     0x04
#define	   LCDVOLTREGOFF    0x00
#define	   LCDVOLTFOLLOWON  0x02
#define	   LCDVOLTFOLLOWOFF 0x00
#define	   LCDVOLTBOOSTON   0x01
#define	   LCDVOLTBOOSTOFF  0x00

// System set command
//   D3-N2, D2-N1, D0-CG
//   N2/1 -> 0,0  2 lines
//   N2/1 -> 0,1  3 lines
//   N2/1 -> 1,0  4 lines
//   CG   -> 1 CGRAM used, 0 not used
#define    LCDSYSTEMSET  0x60
#define    LCD4LINE		 0x08
#define    LCDCGON		 0x01

// Electronic volume setting
//   D3-0 - set display brightness
#define    LCDVOLREG 0x70  

// Set address of DDRAM, CGRAM or symbol register
#define    LCDRAMADDRESS  0x80 

//  LCD chip select is active low
#define	   CS       1
//  LCD Command/Data  0 - command, 1 data
#define    LA0      2
//  LCD reset line - active high to reset
#define    LCDRESET 0
//  LCD Serial Data Line
#define    LCD_SI   2
//  LCD Serial Clock
#define    LCD_SCLK 3

#define    LCDCONTROL PORTB
#define    LCDDATA    PORTC
#define    LINE1 	  0x30
#define	   LINE2	  0x40
#define	   LINE3	  0x50
#define	   LINE4	  0x60

#define    TAB		  0x09
#define    CRLF       0x10
#define    TITLE      0x01

// LCD defines
#define  LCD_BATTERY_FULL    6
#define  LCD_BATTERY_THREEQUARTER 5 
#define  LCD_BATTERY_HALF    4
#define  LCD_BATTERY_QUARTER 3
#define  LCD_BATTERY_EMPTY   2
#define  LCD_BATTERY_OFF     1

#define  LCD_ANTENNA_FULL    6
#define  LCD_ANTENNA_THREEQUARTER  5
#define  LCD_ANTENNA_HALF    4
#define  LCD_ANTENNA_QUARTER 3 
#define  LCD_ANTENNA_EMPTY   2
#define  LCD_ANTENNA_OFF     1

#define  LCD_PHONE_ON		 1
#define  LCD_SLASHPHONE		 2
#define  LCD_PHONE_OFF		 3

#define  LCD_PHONE_SYMBOL      0x02
#define  LCD_BIGANTENNA_SYMBOL 0x08

#define  NO_ACTIVE_STATE 0
#define  ANTENNA_STATE  1
#define  BATTERY_STATE  2
#define  VSYM_STATE     3
#define  DSYM_STATE     4
#define  RSYM_STATE     5
#define  ENVELOPE_STATE 6
#define  PHONE_STATE    7
#define  TWI_ADDRESS_STATE 8
#define  LCD_SYMBOL_STATE 9
#define  LED_BLINK_STATE 10
#define  VOLTAGE_STATE  11

const unsigned char line1chars[] = {'S','m','a','r','t','L','C','D',' ',' ',' ',' ',0x00};
const unsigned char line2chars[] = {'V','e','r','s','i','o','n',' ','0','.','8','9',0x00};
const unsigned char line3chars[] = {'S','O','C',' ','R','o','b','o','t','i','c','s',0x00};
const unsigned char line4chars[] = {'U','a','r','t',' ','I','2','C',' ','O','n',' ',0x00};
const unsigned char line11chars[] = {'U','a','r','t',' ','1','9','2','0','0',',','8',0x00};
const unsigned char line12chars[] = {'I','2','C',' ','A','d','d',' ','0','x',' ',' ',0x00};
const unsigned char line13chars[] = {'S','e','n','d',' ','E','S','C',' ','c',' ',' ',0x00};
const unsigned char line14chars[] = {'f','o','r',' ','t','e','x','t',' ','m','o','d',0x00};
const unsigned char line21chars[] = {'T','e','x','t',' ','D','i','s','p','l','a','y',0x00};
const unsigned char line22chars[] = {'a','c','t','i','v','a','t','e','d',' ',' ',' ',0x00};
const unsigned char line23chars[] = {'P','r','e','s','s',' ','k','e','y',' ','3',' ',0x00};
const unsigned char line24chars[] = {'t','o',' ','e','x','i','t',' ',' ',' ',' ',' ',0x00};
unsigned char line1[13],line2[13],line3[13],line4[13];
unsigned char cursorposition,cursormax,rowptr,colptr,lineptr,escmode;
unsigned char lcdphoneantennasymbols,lcdstate,symbolmode,voltagevolume;

extern struct TWI_S_statusReg TWI_statusReg;

void bytetoint(unsigned char byte)
{
 	 unsigned char tens,ones;
	 
	 tens = byte/10;
	 ones = byte%10;
	 hexbyte[0] = tens + '0';
	 hexbyte[1] = ones + '0';
	 hexbyte[2] = 0x00;
}

void   LCDWriteSerialByte(unsigned char inchar)
{
 	unsigned char i,mask;

	mask = 0x80;
	for(i=0; i<8; i++) {
	    ClrBit(LCDDATA,LCD_SCLK);
		if((inchar&mask)) SetBit(LCDDATA,LCD_SI);
		else ClrBit(LCDDATA,LCD_SI);
		mask >>= 1;
		SetBit(LCDDATA,LCD_SCLK);
	    }
}

void   LCDprintchar(unsigned char inchar)
{
	LCDCONTROL |= BIT(LA0); // raise A0 high
	LCDCONTROL &= ~BIT(CS); // set CS low
	ShortDelay(1);
	LCDWriteSerialByte(inchar);
	LCDCONTROL |= BIT(CS); // release CS
	ShortDelay(2);	
}

void   LCDwritecommand(unsigned char inchar)
{ 		
	LCDCONTROL &= ~BIT(LA0); /* set A0 low */
	LCDCONTROL &= ~BIT(CS);  /* set RS low */
	ShortDelay(1);
	LCDWriteSerialByte(inchar);
	LCDCONTROL |= BIT(CS); // release CS
	ShortDelay(2);
}

void   LCDDisplayChar(unsigned char address, unsigned char inchar)
{
 	LCDwritecommand(LCDRAMADDRESS|address);
	LCDprintchar(inchar);
}

void LCD_Write(unsigned char address,unsigned char data)
{
	LCDwritecommand(LCDRAMADDRESS|address);
	LCDprintchar(data);
}

void   LCDdisplays(unsigned char address, char *instrg)
{
	char i,j;

	i = 0;
	LCDwritecommand(LCDRAMADDRESS|address);
	while(instrg[i]!=0x00) LCDprintchar(instrg[i++]);
}

void   LCDprints(char *instrg)
{
	char i,j;

	i = 0;
	while(instrg[i]!=0x00) {
		if(cursorposition>=cursormax) {
		   cursorposition = 0;
		   return ; // or wrap line
		   }
		else  {
			  if(instrg[i]==LF) LCDwritecommand(LCDCLR);
			  else if(instrg[i]==CR) LCDwritecommand(LCDHOME);
			  else if(instrg[i]==TAB) for(j=0; j<6; j++) LCDprintchar(0x20);
			  else LCDprintchar(instrg[i++]);
			  cursorposition++;
			  }
		}
}

void LCD_Symbols(unsigned char state)
{ 	 
	 int i;
	 
	 if(state==ON) {
	 	 LCDwritecommand(LCDRAMADDRESS|0x70);
	 	 for(i=0; i<12; i++)  LCDprintchar(0x0b);
		 }
	 else if(state==OFF) {
	 	 LCDwritecommand(LCDRAMADDRESS|0x70);
	 	 for(i=0; i<12; i++)  LCDprintchar(0x00);
		 }
}

void SetBuffers(unsigned char inchar)
{
 	 unsigned char i;
	 
	 for(i=0; i<12; i++) {
	     line1[i] = inchar;
		 line2[i] = inchar;
		 line3[i] = inchar;
		 line4[i] = inchar;
		 }
	 line1[12] = line2[12] = line3[12] = line4[12] = 0;
}

void Clear_LCD()
{
 	 int i;
	 
	 // Clear 4x12 display
	 for(i=0; i<12; i++) LCD_Write(LCDRAMADDRESS|0x30+i,' ');
	 for(i=0; i<12; i++) LCD_Write(LCDRAMADDRESS|0x40+i,' ');
	 for(i=0; i<12; i++) LCD_Write(LCDRAMADDRESS|0x50+i,' ');
	 for(i=0; i<12; i++) LCD_Write(LCDRAMADDRESS|0x60+i,' ');
	 
	 // Clear buffers
	 SetBuffers(' ');
	 
	 // Clear all symbols
	 LCD_Symbols(OFF);
}


/******************************************
  LCD interface control logic:
       CS -  0 select device
       R/W - 0 WR, 1 RD,
       LA0 - 0 command, 1 data
 ******************************************/
void Init_LCD()
{
     lcdchiptype = LCDCHIPE1007;  // if lcd display is E1007
	 
	 SetBit(PORTB,PINB6);  // set clock high
	 // Reset LCD - hold in reset for 10usec
	 LCDCONTROL &= ~BIT(LCDRESET);  // Set Reset line low
	 ShortDelay(200);
	 LCDCONTROL |= BIT(LCDRESET);  // Set Reset line high
	 ShortDelay(50);
	
	// Static Icon off - display on - RF circuit on - clear RAM
	// Set display volume

// System set command
//   D3-N2, D2-N1, D0-CG
//   N2/1 -> 0,0  2 lines
//   N2/1 -> 0,1  3 lines
//   N2/1 -> 1,0  4 lines
//   CG   -> 1 CGRAM used, 0 not used
     LCDwritecommand(LCDSYSTEMSET|LCD4LINE|LCDCGON);

// Static Display Command
//   D1/0 - 0,0  Display OFF
//   D1/0 - 0,1  1-2Hz blink
//   D1/0 - 1,0  3-4Hz blink
//   D1/0 - 1,1  Display ON
	 LCDwritecommand(LCDSTATICDISPLAY|LCDDISPLAYON);
	 
// Display control command
//   D3-C, D2-B, D1-DC, D0-D
//   C  -> 1 cursor on, 0 off
//   DC -> 1 double cursor on, 0 off
//   B  -> 1 blink on, 0 off
//   D  -> 1 display on, 0 off	 
//	 LCDwritecommand(LCDDISPLAY|LCDCURSORON|LCDCURSORBLINK|LCDCURSORDISPLAYON);
	 LCDwritecommand(LCDDISPLAY|LCDCURSORDISPLAYON);

// Power Save Command
//   D1-PS, D0-O
//   PS -> 1 power save on, 0 off
//   O  -> 1 oscillator circuit on, 0 off
     LCDwritecommand(LCDPOWERSAVE|LCDPOWERSAVEOFF|LCDOSCON);

	 // Set data pointer to DDRAM
	 LCDwritecommand(LCDRAMADDRESS|0x30);

	 // Wait 20ms for power supply to stabilize
	 ShortDelay(30000);	 

	 // Power supply control
	 //   D2-VC, D1-VF, D0-P
	 //   VC -> 1 voltage regulator on, 0 off
	 //   VF -> 1 voltage follower on, 0 off
	 //   P  -> 1 voltage booster on, 0 off
	 LCDwritecommand(LCDPOWERSUPPLY|LCDVOLTREGON|LCDVOLTFOLLOWON|LCDVOLTBOOSTON);
//	 LCDwritecommand(LCDPOWERSUPPLY|LCDVOLTREGON|LCDVOLTFOLLOWON);

	 // Electronic volume setting
	 //   D3-0 - set display brightness
	 // TBD - real VDD and adjust Voltage Volume accordingly
	 voltagevolume = 0x09;
     LCDwritecommand(LCDVOLREG|voltagevolume); 

	 // move cursor to home position command
	 LCDwritecommand(LCDHOME); 
 
 	 lcdphoneantennasymbols = 0;
	 cursormax = 12;
	 cursorposition = 0;
	 rowptr = 1;
	 colptr = 0;
	 lineptr = LINE1;
	 lcdstate = NO_ACTIVE_STATE;
	 symbolmode = NO_ACTIVE_STATE;
}

void DisplayLCDBuffers()
{
	 LCDdisplays(LINE1,line1);
   	 LCDdisplays(LINE2,line2);
   	 LCDdisplays(LINE3,line3);
   	 LCDdisplays(LINE4,line4);	 
}

void LoadSignon()
{
 	 unsigned char i;
	 
	 cstrcpy(line1,line1chars);
	 cstrcpy(line2,line2chars);
	 cstrcpy(line3,line3chars);
	 cstrcpy(line4,line4chars);
}

void LoadSignon2()
{
 	 unsigned char i;
	 
	 cstrcpy(line1,line11chars);
	 cstrcpy(line2,line12chars);
	 cstrcpy(line3,line13chars);
	 cstrcpy(line4,line14chars);
}

void LoadSignon3()
{
 	 unsigned char i;
	 
	 cstrcpy(line1,line21chars);
	 cstrcpy(line2,line22chars);
	 cstrcpy(line3,line23chars);
	 cstrcpy(line4,line24chars);
}

void ShiftUpLine()
{
 	 unsigned char i;
	 
	 for(i=0; i<12; i++) {
	     line1[i] = line2[i];
		 line2[i] = line3[i];
		 line3[i] = line4[i];
		 line4[i] = ' ';
		 }
     DisplayLCDBuffers();
	 cursorposition = 0;
}

void PutCharinBuffer(unsigned char row,unsigned char address,unsigned char inchar)
{
    if(row==0)  line1[address] = inchar;
	else if(row==1)  line2[address] = inchar;
	else if(row==2)  line3[address] = inchar;
	else if(row==3)  line4[address] = inchar;
}

void LCD_Dsymbol(unsigned char state)
{
    if(state==ON) LCD_Write(0x70,0x08);
	else if(state==OFF) LCD_Write(0x70,0);
}

void LCD_Rsymbol(unsigned char state)
{
    if(state==ON) LCD_Write(0x75,0x02);
	else if(state==OFF) LCD_Write(0x75,0);
}

void LCD_Vsymbol(unsigned char state)
{
    if(state==ON) LCD_Write(0x78,0x08);
	else if(state==OFF) LCD_Write(0x78,0);
}

void LCD_Phone(unsigned char state)
{
    if(state==LCD_PHONE_ON) {
	    lcdphoneantennasymbols |= LCD_PHONE_SYMBOL;
		LCD_Write(0x73,lcdphoneantennasymbols);
		LCD_Write(0x74,0);
	    }
	else if(state==LCD_SLASHPHONE) {
	    lcdphoneantennasymbols |= LCD_PHONE_SYMBOL;
		LCD_Write(0x73,lcdphoneantennasymbols);
		LCD_Write(0x74,0x08);
	    }
	else if(state==LCD_PHONE_OFF) {
	    lcdphoneantennasymbols &= ~LCD_PHONE_SYMBOL;
		LCD_Write(0x73,lcdphoneantennasymbols);
		LCD_Write(0x74,0);
	    }
}

void LCD_Envelope(unsigned char state)
{
    if(state==ON) LCD_Write(0x76,0x02);
	else if(state==OFF) LCD_Write(0x76,0);
}

void LCD_Battery(unsigned char level)
{
 	 if(level==LCD_BATTERY_EMPTY)  {
	     LCD_Write(0x79,0x02);
	     LCD_Write(0x7a,0);
	     LCD_Write(0x7b,0);
		 }
 	 if(level==LCD_BATTERY_QUARTER)  {
	     LCD_Write(0x79,0x02);
	     LCD_Write(0x7a,0);
	     LCD_Write(0x7b,0x08);
		 }
 	 if(level==LCD_BATTERY_HALF)  {
	     LCD_Write(0x79,0x02);
	     LCD_Write(0x7a,0x02);
	     LCD_Write(0x7b,0x08);
		 }
 	 if(level==LCD_BATTERY_THREEQUARTER)  {
	     LCD_Write(0x79,0x02);
	     LCD_Write(0x7a,0x02);
	     LCD_Write(0x7b,0x0b);
		 }
 	 if(level==LCD_BATTERY_FULL)  {
	     LCD_Write(0x79,0x02);
	     LCD_Write(0x7a,0x0b);
	     LCD_Write(0x7b,0x0b);
		 }
 	 if(level==LCD_BATTERY_OFF)  {
	     LCD_Write(0x79,0);
	     LCD_Write(0x7a,0);
	     LCD_Write(0x7b,0);
		 }
}

void LCD_Antenna(unsigned char level)
{
 	 if(level==LCD_ANTENNA_EMPTY)  {
	     lcdphoneantennasymbols &= ~LCD_BIGANTENNA_SYMBOL;
	     LCD_Write(0x71,0x08);
	     LCD_Write(0x72,0);
	     LCD_Write(0x73,lcdphoneantennasymbols);
		 }
  	 if(level==LCD_ANTENNA_QUARTER)  {
	     lcdphoneantennasymbols &= ~LCD_BIGANTENNA_SYMBOL;
	     LCD_Write(0x71,0x0b);
	     LCD_Write(0x72,0);
	     LCD_Write(0x73,lcdphoneantennasymbols);
		 }
 	 if(level==LCD_ANTENNA_HALF)  {
	     lcdphoneantennasymbols &= ~LCD_BIGANTENNA_SYMBOL;
	     LCD_Write(0x71,0x0b);
	     LCD_Write(0x72,0x08);
	     LCD_Write(0x73,lcdphoneantennasymbols);
		 }
 	 if(level==LCD_ANTENNA_THREEQUARTER)  {
	     lcdphoneantennasymbols &= ~LCD_BIGANTENNA_SYMBOL;
	     LCD_Write(0x71,0x0b);
	     LCD_Write(0x72,0x0b);
	     LCD_Write(0x73,lcdphoneantennasymbols);
		 }
 	 if(level==LCD_ANTENNA_FULL)  {
	     lcdphoneantennasymbols |= LCD_BIGANTENNA_SYMBOL;
	     LCD_Write(0x71,0x0b);
	     LCD_Write(0x72,0x0b);
	     LCD_Write(0x73,lcdphoneantennasymbols);
		 }
 	 if(level==LCD_ANTENNA_OFF)  {
	     lcdphoneantennasymbols &= ~LCD_BIGANTENNA_SYMBOL;
	     LCD_Write(0x71,0);
	     LCD_Write(0x72,0);
	     LCD_Write(0x73,lcdphoneantennasymbols);
		 }
}

unsigned char GetRowfromLine(unsigned char line)
{
 	 if(line==LINE1) return 0;
	 if(line==LINE2) return 1;
	 if(line==LINE3) return 2;
	 if(line==LINE4) return 3;
	 return 0;
}

void SendCharToDisplay(unsigned char inchar)
{
	if(cursorposition>11) {
		if(lineptr==LINE4) ShiftUpLine();
		else {
			lineptr += 0x10;
			cursorposition = 0;
			}
		}				
	LCDDisplayChar(lineptr+cursorposition,inchar);
	PutCharinBuffer(GetRowfromLine(lineptr),cursorposition,inchar);
	cursorposition++;
}

void IncrementLinePtr()
{
    if(lineptr==LINE1)  lineptr = LINE2;
	else if(lineptr==LINE2) lineptr = LINE3;
	else if(lineptr==LINE3) lineptr = LINE4;
}

void ParseEscSubCommand(unsigned char inchar)
{
 	 switch (symbolmode) {
		case TWI_ADDRESS_STATE:
		    if(inchar==LCD_PHONE_ON) LCD_Phone(LCD_PHONE_ON);
			else if(inchar==LCD_SLASHPHONE) LCD_Phone(LCD_SLASHPHONE);
			else if(inchar==LCD_PHONE_ON) LCD_Phone(LCD_PHONE_OFF);
			break;
		case LED_BLINK_STATE:
		    if(inchar==LCD_PHONE_ON) LCD_Phone(LCD_PHONE_ON);
			else if(inchar==LCD_SLASHPHONE) LCD_Phone(LCD_SLASHPHONE);
			else if(inchar==LCD_PHONE_ON) LCD_Phone(LCD_PHONE_OFF);
			break;
		case VOLTAGE_STATE:
		    if(inchar=='o') LCD_Vsymbol(ON);
			else if(inchar=='f') LCD_Vsymbol(OFF);
			break;
		}
 	 lcdstate = NO_ACTIVE_STATE;
}

void ParseSymbolCommand(unsigned char inchar)
{
 	 switch (symbolmode) {
		case ANTENNA_STATE:
		    LCD_Antenna(inchar);
			break;
		case BATTERY_STATE:
		    LCD_Battery(inchar);
			break;
		case VSYM_STATE:
		    if(inchar=='o') LCD_Vsymbol(ON);
			else if(inchar=='f') LCD_Vsymbol(OFF);
			break;
		case DSYM_STATE:
		    if(inchar=='o') LCD_Dsymbol(ON);
			else if(inchar=='f') LCD_Dsymbol(OFF);
			break;
		case RSYM_STATE:
		    if(inchar=='o') LCD_Rsymbol(ON);
			else if(inchar=='f') LCD_Rsymbol(OFF);
			break;
		case ENVELOPE_STATE:
		    if(inchar=='o') LCD_Envelope(ON);
			else if(inchar=='f') LCD_Envelope(OFF);
			break;
		case PHONE_STATE:
		    LCD_Phone(inchar);
			break;
	     }
 	 lcdstate = NO_ACTIVE_STATE;
	 symbolmode = NO_ACTIVE_STATE;
}

void ParseEscCommand(unsigned char inchar)
{
    switch (inchar) {
	    case 'c':
		    Clear_LCD();
			textdisplaymode = ACTIVE;
			lineptr = LINE1;
			cursorposition = 0;
			escmode = OFF;
			break;
		case 'x':
		    Clear_LCD();
			textdisplaymode = INACTIVE;
			lineptr = LINE1;
		    cursorposition = 0;
			escmode = OFF;
			break;
		case '1':
		    lineptr = LINE1;
		    cursorposition = 0;
			escmode = OFF;
			break;
		case '2':
		    lineptr = LINE2;
			cursorposition = 0;
			escmode = OFF;
			break;
		case '3':
		    lineptr = LINE3;
			cursorposition = 0;
			escmode = OFF;
			break;
		case '4':
		    lineptr = LINE4;
			cursorposition = 0;
			escmode = OFF;
			break;
		case 'o':
			//SendCharToDisplay('o');
		    LCD_LED_Power(ON);
			escmode = OFF;
			break;
		case 'f':
			//SendCharToDisplay('f');
		    LCD_LED_Power(OFF);
			escmode = OFF;
			break;
		case 's':
		    lcdstate = LCD_SYMBOL_STATE;
			break;
		case 'v':
		    lcdstate = VOLTAGE_STATE;
			break;
		case 'l':
		    lcdstate = LED_BLINK_STATE;
			break;
		case 't':
		    lcdstate = TWI_ADDRESS_STATE;
			break;
		default:
		    escmode = OFF;
			break;
		}
	escmode = OFF;
}

void ParseChar(unsigned char inchar)
{
	if(escmode==ON) {
		ParseEscCommand(inchar);
		}
	else {
	    if(symbolmode)  ParseSymbolCommand(inchar);
	    else if(lcdstate==LCD_SYMBOL_STATE) symbolmode = inchar;
		else if(lcdstate) ParseEscSubCommand(inchar);
		else {
	      switch (inchar) {
		    case ESC:
				escmode = ON;  
				break;
		    case CR:
				cursorposition = 0; 
				break;
		    case LF:
				if(lineptr==LINE4) ShiftUpLine();
 				else IncrementLinePtr();
				break;
		    case 0x01:
				LCD_Vsymbol(ON); 
				break;
		    case 0x02:
				LCD_Vsymbol(OFF);  
				break;
		    case 0x03:
				LCD_Dsymbol(ON); 
				break;
		    case 0x04:
				LCD_Dsymbol(OFF);  
				break;
		    case 0x05:
				LCD_Rsymbol(ON); 
				break;
		    case 0x06:
				LCD_Rsymbol(OFF);  
				break;
		    case 0x07:
				LCD_Envelope(ON); 
				break;
		    case 0x08:
				LCD_Envelope(OFF);  
				break;
			default:
			    SendCharToDisplay(inchar);
				break;
			}
		  }
		}
}

void ParseStrg(unsigned char *instrg)
{
 	 unsigned char i;
	 
	 for(i=0; i<strlen(instrg); i++)  ParseChar(instrg[i]);
}

void PutStringinBuffer(unsigned char row,unsigned char column,char *strg)
{	 
	 unsigned char i;
	 
	 for(i=0; i<strlen(strg); i++)  PutCharinBuffer(row,column+i,strg[i]);
}

unsigned char GetLine(unsigned char row)
{
    if(row==0) return LINE1;
	if(row==1) return LINE2;
	if(row==2) return LINE3;
	if(row==3) return LINE4;
	return LINE1;
}

void Display_Voltage_Volume(unsigned char row,unsigned char column)
{
 	 char strg[5];
 	 int analogvalue;
	 
	 PutStringinBuffer(row,column,"Volt v=");
	 bytetohex(voltagevolume);
	 strcpy(strg,hexbyte);
	 PutStringinBuffer(row,column+7,hexbyte);
}

void Int2strg(int intvalue)
{
 	 int  tenthousands, thousands, hundreds, tens;

	 thousands = intvalue/1000;
	 intvalue -= (thousands*1000);
	 hundreds = intvalue/100;
	 intvalue -= (hundreds*100);
	 tens = intvalue/10;
	 numstrg[0] = '0' + hundreds;
	 numstrg[1] = '0' + tens;
	 numstrg[2] = '0' + intvalue%10; 
	 numstrg[3] = ' ';
	 numstrg[4] = ' ';
	 numstrg[5] = 0x00;
}

void Float2strg(int intvalue)
{
 	 int  tenthousands, thousands, hundreds, tens;

	 thousands = intvalue/1000;
	 intvalue -= (thousands*1000);
	 hundreds = intvalue/100;
	 intvalue -= (hundreds*100);
	 tens = intvalue/10;
	 numstrg[0] = '0' + thousands;
	 numstrg[1] = '.';
	 numstrg[2] = '0' + hundreds;
	 numstrg[3] = '0' + tens;
	 numstrg[4] = '0' + intvalue%10; 
	 numstrg[5] = 0x00;
}

void Display_Voltage(unsigned char row,unsigned char column)
{
 	 char strg[6];
 	 int analogvalue,newvoltvol,intvalue;
 	 int  tenthousands, thousands, hundreds, tens;
	 float voltage;
	 
	 PutStringinBuffer(row,column,"Voltage=");
 	 analogvalue = Read_ADC(0);
	 voltage = (float) analogvalue/159.72;
	 intvalue = voltage * 1000; // shift to 1000 2.234 becomes 2234
	 
	 Float2strg(intvalue);
	 
	 PutStringinBuffer(row,column+8,numstrg);
}

void Display_Voltage_Reference(unsigned char row,unsigned char column)
{
 	 char strg[5];
 	 int analogvalue,newvoltvol;
	 
	 PutStringinBuffer(row,column,"Analog0=");
 	 analogvalue = Read_ADC(0);
	  
	 bytetohex((analogvalue>>8)&0xff);
	 strcpy(strg,hexbyte);
	 bytetohex((analogvalue&0x00ff));
	 strcat(strg,hexbyte);
	 PutStringinBuffer(row,column+8,strg);
}

void  Adjust_VoltageVolume()
{
	 if((voltagevolume=(1+(574-Read_ADC(0))/14))>0x0f) 
	     voltagevolume = 0x0f;
	 LCDwritecommand(LCDVOLREG|voltagevolume); 
}


void DisplayTime(unsigned char row,unsigned char column)
{
 	 char strg[9];
	 	
	 //PutStringinBuffer(column,"t=");
	 bytetoint(hours);
	 strcpy(strg,hexbyte);
	 strcat(strg,":");
	 bytetoint(minutes);
	 strcat(strg,hexbyte);
	 strcat(strg,":");
	 bytetoint(seconds);
	 strcat(strg,hexbyte);
	 PutStringinBuffer(row,column,strg);
}

void Display_Display_Parameters()
{
	if(flip2==ON) {
		//LCD_Symbols(ON);
		//Display_Voltage_Reference(0,0);
		Display_Voltage(0,0);
		Display_Voltage_Reference(1,0);
		//Display_Voltage_Volume(1,0);
		DisplayTime(3,0);
		LED_Power(OFF);
		//LCD_LED_Power(ON);
		flip2 = YES;
		//line4[11] = '.';
    	DisplayLCDBuffers();
		}
	else {
		//line4[11] = ' ';
		//LCD_Symbols(OFF);	
    	DisplayLCDBuffers();
		//LCD_LED_Power(OFF);
		LED_Power(ON);
		flip2 = NO;
		}
	mseccountdown2 = 250;
}

void Display_Count()
{
 	char outstrg[12];
	
	rtccountdown1 = 256;
	bytetohex((count>>8)&0xff);
	strcpy(outstrg,hexbyte);
	bytetohex((count&0x00ff));
	strcat(outstrg,hexbyte);
	PutStringinBuffer(2,0,"count=");
	PutStringinBuffer(2,6,outstrg);
	count++;
}

void main()
{
    unsigned char inchar,messageBuf[MESSAGEBUF_SIZE], TWI_slaveAddress;
	unsigned char symbolptr,level,level2,outbyte,voltlevel,outchar,twichars;
    char flip,outstrg[12];
	int i,symbolindex,analogvalue;
	float realnum;

	Init_AVR();
    Init_LCD();
    Clear_LCD();
	
	Adjust_VoltageVolume(); 
//   LCDtest();
   
    TWI_slaveAddress = TWI_SLAVEADDRESS;  // Set TWI slave address
//    TWI_Slave_Initialise(TWI_slaveAddress,TWI_generalCall,TWI_mask);
    TWI_Slave_Initialise(TWI_slaveAddress,0,0);
	bytetohex(TWI_slaveAddress);
	bytetohex(0x26);
	strcpy(outstrg,hexbyte);
	PutStringinBuffer(3,6,outstrg);

//   load_eeprom_settings();  //load all start values in eeprom

    DisplayLCDBuffers();
    rowptr = 4;
    cursorposition = 12;
    lineptr = LINE4;
    escmode = OFF;
    inchar = 0x31;
	flip = NO;
	symbolindex = 0;
	symbolptr = 0x00;
	level2 = level = 1;
	voltlevel = 0;
	count = 0;
	outchar = 0x30;
    LCD_Symbols(OFF);
	//LCD_Antenna(LCD_ANTENNA_FULL);
	//LCD_Battery(LCD_BATTERY_FULL);
	outbyte = 0;

	Adjust_VoltageVolume(); 
	LoadSignon();
	DisplayLCDBuffers();
	lineptr = LINE1;
	cursorposition = 0;	
	Delaymsec(1000);
	
	Adjust_VoltageVolume(); 
	LoadSignon();

	DisplayLCDBuffers();
	lineptr = LINE1;
	cursorposition = 0;	
	Delaymsec(3000);
	Clear_LCD();
	Delaymsec(200);
	
	LoadSignon2();
	bytetohex(TWI_slaveAddress);
	strcpy(outstrg,hexbyte);
	PutStringinBuffer(1,10,outstrg);
	DisplayLCDBuffers();
	lineptr = LINE1;
	cursorposition = 0;	
	Delaymsec(5000);
	Clear_LCD();
	Delaymsec(1000);
	
	//  Output signon message to UART - ESC c turns character
	//  display on
	putfs(signon1);
	putfs(signon11);
	putfs(signon12);
	PutStringinBuffer(3,10,outstrg);
	
	//  Monitor TWI and Uart ports for display data
	while(1) { 
		
		LCD_LED_Power(OFF);
		//LED_Power(OFF);
		
	    if(0) {
	      if(mseccountdown1==0) {
		    if(flip==NO) {
				if(symbolindex==0) {
					LCD_Envelope(ON);
					LCD_Dsymbol(ON);
					LCD_Rsymbol(ON);
					LCD_Vsymbol(ON);
					LCD_Phone(level2++);
					LCD_Battery(level);
					LCD_Antenna(level++);
					if(level==7) level = 1;
					if(level2==4) level2 = 1;
					}
		        DisplayLCDBuffers();
				flip = YES;
		        }
		    else {
				LCD_Envelope(OFF);
				LCD_Dsymbol(OFF);
				LCD_Rsymbol(OFF);
				LCD_Vsymbol(OFF);
				DisplayLCDBuffers();
				flip = NO;
			    }
		    mseccountdown1 = 500;
			}
			}

		if((CheckKeyState(KEY3))&&(textdisplaymode==ACTIVE)) {
		    textdisplaymode = INACTIVE;
			}
		
		if(mseccountdown4==0) {
		    Adjust_VoltageVolume();
			mseccountdown4 = 1;
			}
		
		if(textdisplaymode==INACTIVE) {
		    if(rtccountdown1==0) Display_Count();
			
		    if(mseccountdown2==0) Display_Display_Parameters();
		   
		    if(0) {
		        //if(mseccountdown3==0) {
		        SendCharToDisplay(outchar++);
		        if(outchar>0x7a) outchar = 0x30;
		        mseccountdown3 = 50;
		        }
			}
   
   		if(textdisplaymode==INACTIVE) {
		    if(CheckKeyState(KEY1)) {
			    // Display 3-axis acceleration
				
			    while(CheckKeyState(KEY1)) {
				    enableAccelerometer(ON);
				    setAccelerometerRange(ACCELEROMETER_1g5_SENSITIVITY);
					analogvalue = readAccelerometer(X_AXIS);
					Int2strg(analogvalue);
					PutStringinBuffer(0,0,"X Axis=");
					PutStringinBuffer(0,7,numstrg);
				    analogvalue = readAccelerometer(Y_AXIS);
					Int2strg(analogvalue);
					PutStringinBuffer(1,0,"Y Axis=");
					PutStringinBuffer(1,7,numstrg);
				    analogvalue = readAccelerometer(Z_AXIS);
					Int2strg(analogvalue);
					PutStringinBuffer(2,0,"Z Axis=");
					PutStringinBuffer(2,7,numstrg);
				    line4[8] = 'p';
					DisplayLCDBuffers();
				    }
				LCD_Symbols(OFF);
				Clear_LCD();
				}
			else line4[8] = ' ';
			if(CheckKeyState(KEY2)) line4[9] = 'p';
			else line4[9] = ' ';
			if(CheckKeyState(KEY3)) {
				if(lcdledpower==ON) {
				    LCD_LED_Power(OFF);
					lcdledpower = OFF;
					}
				else {
				    LCD_LED_Power(ON);
					lcdledpower = ON;
				    }
			    line4[10] = 'p';
				}
			else line4[10] = ' ';
			if(CheckKeyState(KEY4)) {
			    line4[11] = 'p';
			    LoadSignon3();
			    DisplayLCDBuffers();
			    Delaymsec(3000);
			    lineptr = LINE1;
			    cursorposition = 0;	
			    Clear_LCD();
				ParseChar(ESC);
				ParseChar('c');
				}
			else line4[11] = ' ';
		}
		   
		//  Check Uart RX for display commands
		if((inchar=getcharnw())!=0)  {
		    ParseChar(inchar);
			putchar(inchar);
			}
		
		//  Check TWI Input for display commands
	    if(1) {
		    // See if any data arrived on TWI
			if((twichars=TWI_Get_Data_From_Transceiver(messageBuf,4))>0) {
			    for(i=0; i<twichars; i++) ParseChar(messageBuf[i]);
			    }
	         }
	    }
}
